package com.develop.auth.ui

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.lifecycle.MutableLiveData
import com.develop.auth.databinding.FragmentMemberProfileBinding
import com.develop.auth.viewmodel.MemberViewModel
import com.develop.base.ext.navigateTo
import com.develop.base.ext.resId2Dimension
import com.develop.base.ext.setVisible
import com.develop.base.mvvm.BaseBVMFragment
import com.develop.base.util.ThreadUtils
import com.develop.common.data_repo.FoodDataProvider
import com.develop.common.data_repo.db.entity.UserInfo
import com.develop.common.data_repo.net.model.response.User
import com.develop.common.dialog.AgeSelectDialog
import com.develop.common.dialog.GenderSelectDialog
import com.develop.common.router.Screens
import com.develop.common.ui.CommonBVMFragment

class MemberProfileFragment : CommonBVMFragment<FragmentMemberProfileBinding, MemberViewModel>() {

    private var selectAge = 0
    private var selectGender = 2


    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?
    ): FragmentMemberProfileBinding {
        return FragmentMemberProfileBinding.inflate(inflater, container, false)
    }

    override fun onPostCreateView() {
        super.onPostCreateView()
        viewModel.queryUserInfo()
        binding.tvChooseAge.setOnClickListener {
            AgeSelectDialog(hostActivity) { index, change ->
                selectAge = index + 20
                binding.tvChooseAge.text = selectAge.toString()
                binding.tvChooseAge.setTextColor(Color.BLACK)
            }.show()
        }

        binding.tvChooseSex.setOnClickListener {
            GenderSelectDialog {
                selectGender = if (it) 2 else 1
                if (selectGender == 2) {
                    binding.tvChooseSex.text = "Male"
                } else {
                    binding.tvChooseSex.text = "Female"
                }
                binding.tvChooseSex.setTextColor(Color.BLACK)
            }.show(parentFragmentManager, "gender")
        }
        binding.tvSave.setOnClickListener {
            saveProfile(
                binding.etNickname.text.toString(),
                selectAge,
                selectGender,
                viewModel.chooseAvatar.value ?: 0
            )
        }
        binding.tvSkipForNow.setOnClickListener {
            navigateTo(Screens.Main.ENTRANCE_CHOSEN)
        }
        binding.tvDestroyAccount.setOnClickListener {

        }

        if (viewModel.enterFromHome) {
            (binding.tvTitle.layoutParams as ViewGroup.MarginLayoutParams).let {
                it.topMargin = dp53
                binding.tvTitle.requestLayout()
            }
            (binding.tvSave.layoutParams as ViewGroup.MarginLayoutParams).let {
                it.topMargin = dp101
                binding.tvSave.requestLayout()
            }
            (binding.tvSkipForNow.layoutParams as ViewGroup.MarginLayoutParams).let {
                it.topMargin = dp16
                binding.tvSkipForNow.requestLayout()
            }
            binding.tvSkipForNow.text = "Sign out"
            binding.tvDestroyAccount.setVisible()
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        viewModel.apply {
            userInfoLiveData.observe(viewLifecycleOwner) {
                it?.apply {
                    selectAge = userAge
                    selectGender = userGender?.toInt() ?: 2
                    binding.etNickname.setText(userName ?: "")
                    viewModel.chooseAvatar.value = userAvatar?.toInt()
                }
            }
            perfectInfoLiveData.observe(viewLifecycleOwner) {
                dismissPlainDialog()
                it.msg?.apply {
                    showToast(this)
                }
            }
        }
    }

    private fun saveProfile(
        nickname: String?, age: Int, sex: Int, portrait: Int
    ) {
        if (nickname.isNullOrEmpty()) {
            showToast("Please fill in nickname")
            return
        }
        if (age == 0) {
            showToast("Please fill in age")
            return
        }
        showPlainDialog(true)
        viewModel.perfectInfo(nickname, sex.toString(), portrait.toString(), age.toString())
    }

    override fun createViewModel(): MemberViewModel {
        return getViewModelOfActivity(MemberViewModel::class.java)
    }

    companion object {
        var dp53 = (com.develop.base.R.dimen.convert_53px).resId2Dimension().toInt()
        var dp101 = (com.develop.base.R.dimen.convert_101px).resId2Dimension().toInt()
        var dp16 = (com.develop.base.R.dimen.convert_16px).resId2Dimension().toInt()
    }
}