package com.develop.main.ui

import android.os.Bundle
import android.view.LayoutInflater
import androidx.lifecycle.MutableLiveData
import com.alibaba.android.arouter.facade.annotation.Route
import com.develop.base.ext.globalApp
import com.develop.common.data_repo.FoodDataProvider
import com.develop.base.ext.navigateTo
import com.develop.base.mvvm.BaseBindingActivity
import com.develop.base.util.MMkvUtils
import com.develop.common.dialog.LoadingDialog
import com.develop.common.router.Screens
import com.develop.common.tag.FIRST_IN
import com.develop.common.tag.MODE_ENTRANCE
import com.develop.common.tag.Mode
import com.develop.common.tag.Recipes
import com.develop.common.ui.CommonBindingActivity
import com.develop.main.databinding.ActivityModeEntranceBinding

@Route(path = Screens.Main.ENTRANCE_CHOSEN)
class ModeEntranceActivity : CommonBindingActivity<ActivityModeEntranceBinding>() {

    private val loadingDialog by lazy {
        LoadingDialog()
    }

    var prepareDataLiveData = MutableLiveData<Boolean>()
    override fun createViewBinding(inflater: LayoutInflater): ActivityModeEntranceBinding {
        return ActivityModeEntranceBinding.inflate(inflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        MMkvUtils.save(FIRST_IN, true)
        initListener()
        prepareDataLiveData = FoodDataProvider.getPrepareLiveData() as MutableLiveData<Boolean>
        prepareDataLiveData.observe(this) {
            loadingDialog.removeSelf()
        }
    }

    private fun initListener() {
        binding.apply {
            modeLayout.setOnClickListener {
                val bundle = Bundle()
                bundle.putInt(MODE_ENTRANCE, Mode)
                navigateTo(Screens.Main.HOME) {
                    with(bundle)
                }
                finish()
            }
            recipesLayout.setOnClickListener {
                if (FoodDataProvider.checkDataExit()) {
                    val bundle = Bundle()
                    bundle.putInt(MODE_ENTRANCE, Recipes)
                    navigateTo(Screens.Main.HOME) {
                        with(bundle)
                    }
                    finish()
                } else {
                    showLoading()
                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        loadingDialog.removeSelf()
    }

    private fun showLoading() {
        loadingDialog.loadingTitle = "数据加载中"
        loadingDialog.showDialog(supportFragmentManager, "Loading")
    }

}