package com.develop.main.ui


import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import androidx.appcompat.widget.AppCompatTextView
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.RecyclerView
import com.develop.base.ext.navigateTo
import com.develop.base.ext.resId2Dimension
import com.develop.base.ext.updateText
import com.develop.base.widgets.CommonPopupWindow
import com.develop.common.data_repo.db.*
import com.develop.common.data_repo.db.entity.DevRecipeCategory
import com.develop.common.databinding.FragmentCommeListBinding
import com.develop.common.router.Screens
import com.develop.common.tag.IS_LIKE_TAG
import com.develop.common.tag.NUMBER_TAG
import com.develop.common.tag.RECIPES_EDITION_TAG
import com.develop.common.tag.SOURCE_TAG
import com.develop.common.ui.CommonBVMFragment
import com.develop.common.widget.*
import com.develop.main.R
import com.develop.main.adapter.HomeBannerAdapter
import com.develop.main.databinding.ItemFilterAndSortViewBinding
import com.develop.main.viewmodel.HomeViewModel
import com.drake.brv.BindingAdapter
import com.drake.brv.utils.linear
import com.drake.brv.utils.models
import com.drake.brv.utils.setup
import com.youth.banner.Banner

class RecipesFragment : CommonBVMFragment<FragmentCommeListBinding, HomeViewModel>() {

    private var filterSortSearchView: HomeFilterSortSearchView? = null

    private var sortRv: RecyclerView? = null
    private var filterRv: RecyclerView? = null

    private var categoryDataList = mutableListOf<DevRecipeCategory>()
    private var sortDataList = mutableListOf<FilterSortModel>()
    private var currentHotWord = ""
    private var categoryCode: String = ""

    private val filterPopupWindow by lazy {
        CommonPopupWindow.ViewBuilder<FilterSortViewLayout>().width(dp626).height(dp768)
            .outsideTouchable(true).focusable(true).alpha(0.5f).clippingEnabled(false)
            .view(FilterSortViewLayout(hostActivity)).intercept { popupWindow, view ->

            }.onShowBefore { popupWindow, view ->
                filterRv = view.findViewById(com.develop.common.R.id.filter_recycle)
                filterRv?.apply {
                    linear()
                    setup {
                        addType<DevRecipeCategory>(R.layout.item_filter_and_sort_view)
                        onBind {
                            val model = getModel<DevRecipeCategory>()
                            bindFilterSortView(adapter, this, model.name ?: "", true)
                        }
                    }
                }?.models = categoryDataList
            }.build<ViewDataBinding>(hostActivity)
    }

    private val sortPopupWindow by lazy {
        CommonPopupWindow.ViewBuilder<FilterSortViewLayout>().width(dp626)
            .height(RelativeLayout.LayoutParams.WRAP_CONTENT).outsideTouchable(true).focusable(true)
            .alpha(0.5f).clippingEnabled(false).view(FilterSortViewLayout(hostActivity))
            .intercept { popupWindow, view ->

            }.onShowBefore { popupWindow, view ->
                sortRv = view.findViewById(com.develop.common.R.id.filter_recycle)
                sortRv?.apply {
                    linear()
                    setup {
                        addType<FilterSortModel>(R.layout.item_filter_and_sort_view)
                        onBind {
                            val model = getModel<FilterSortModel>()
                            bindFilterSortView(adapter, this, model.name, false)
                        }
                    }
                }?.models = sortDataList
            }.build<ViewDataBinding>(hostActivity)
    }


    override fun createViewModel(): HomeViewModel {
        return getViewModelOfActivity(HomeViewModel::class.java)
    }

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?
    ): FragmentCommeListBinding {
        return FragmentCommeListBinding.inflate(inflater, container, false)
    }

    override fun onPostCreateView() {
        super.onPostCreateView()
        initView()
        initLiveDataCallback()
        viewModel.getOnLineCategoryList()
    }

    override fun onResume() {
        super.onResume()
        if (viewModel.recipesType == RecipesType.ONLINE) {
            viewModel.getOnLineCategoryList()
        } else {
            viewModel.queryRecipesCategory()
        }
    }

    private fun initView() {
        sortDataList.addAll(DataFactory.genSortNameList(resources))

        binding.rv.apply {
            linear()
            setup {
                addType<String>(HOME_TITLE_PROVIDER)
                addType<FilterSortModel>(HOME_FUNCTION_PROVIDER)
                addType<FoodContentModel>(HOME_FOOD_PROVIDER)
                addType<HomeBannerListModel>(HOME_BANNER_PROVIDER)
                addType<CommonSearchModel>(HOME_SEARCH_PROVIDER)
                addType<CategoryModel>(HOME_CATEGORY_PROVIDER)
                onBind {
                    when (itemViewType) {
                        HOME_TITLE_PROVIDER -> {
                            findView<AppCompatTextView>(R.id.tv_food_category_name).updateText(
                                getModel()
                            )
                        }
                        HOME_FUNCTION_PROVIDER -> {
                            filterSortSearchView =
                                findView<HomeFilterSortSearchView>(R.id.function_view)
                            filterSortSearchView?.onFunctionCallback = functionCallback
                        }
                        HOME_FOOD_PROVIDER -> {
                            findView<FoodContentView>(R.id.food_content_view)
                                .setFoodContent(getModel())
                        }
                        HOME_BANNER_PROVIDER -> {
                            val bannerAdapter =
                                HomeBannerAdapter(getModel<HomeBannerListModel>().data)
                            bannerAdapter.onBannerItemClickListener = bannerItemClickCallback
                            findView<Banner<*, *>>(R.id.home_banner_view).adapter = bannerAdapter

                        }
                        HOME_SEARCH_PROVIDER -> {
                            findView<CommonSearchLayout>(R.id.common_search_view).apply {
                                updateData(getModel())
                                onHotWordsSelectedListener = hotWordsSelectedCallback
                            }
                        }
                        HOME_CATEGORY_PROVIDER -> {
                            findView<CategoryTabViewLayout>(R.id.category_tab_view).apply {
                                val model = getModel<CategoryModel>()
                                updateCategorySize(
                                    model.allSize, model.recipesSize, model.ingredientSize
                                )
                                onCategoryTabSelectedListener = categoryTabSelectedCallback
                            }
                        }
                    }
                }
            }
        }
        binding.page.apply {
            setEnableRefresh(false)
            onRefresh {
                if (viewModel.recipesType == RecipesType.ONLINE) {
                    viewModel.netFoodLiveData.postValue(true)
                    viewModel.getOnLineCategoryList()
                } else {
                    viewModel.queryLocalRecipes(categoryCode)
                }
            }
            onLoadMore {
                if (viewModel.recipesType == RecipesType.ONLINE) {
                    if (viewModel.currentPage < viewModel.totalPageSize) {
                        viewModel.getOnLineRecipeList(
                            categoryCode, viewModel.currCategoryName, isLoadMore = true
                        )
                    } else {
                        finish(hasMore = false)
                    }
                } else {
                    finish(hasMore = false)
                }
            }
        }


    }


    /**排序/查询/过滤功能按钮回调 */
    private var functionCallback = object : HomeFilterSortSearchView.OnFunctionCallback {
        override fun onFilterClick(filterView: View) {
            filterRv?.models = categoryDataList
            filterPopupWindow.showAsDropDown(filterView, -dp5, 0)
        }

        override fun onSortClick(sortView: View) {
            sortRv?.models = sortDataList
            sortPopupWindow.showAsDropDown(sortView, dp65, 0)
        }

        override fun onSearchClick() {
            //进入搜索页
            if (viewModel.recipesType == RecipesType.ONLINE) {
                viewModel.getOnLineHotTags()
            } else {
                binding.rv.models = DataFactory.genTestSearchDataList()
            }
        }

        override fun onInputSearch(searchContent: String) {
            currentHotWord = searchContent
            if (viewModel.recipesType == RecipesType.ONLINE) {
                viewModel.getOnLineRecipeList("", currentHotWord)
            } else {
                viewModel.queryRecipesByHotWord(currentHotWord, CategoryType.All)
            }
        }

        override fun onBackClick() {
            if (viewModel.recipesType == RecipesType.ONLINE) {
                viewModel.getOnLineCategoryList()
            } else {
                viewModel.queryLocalRecipes(categoryCode)
            }
        }

    }

    /**热词选择回调 */
    private var hotWordsSelectedCallback = object : CommonSearchLayout.OnHotWordsSelectedListener {
        override fun onHotWordsSelected(hotWord: String) {
            currentHotWord = hotWord
            if (viewModel.recipesType == RecipesType.ONLINE) {
                viewModel.getOnLineRecipeList("", currentHotWord)
            } else {
                viewModel.queryRecipesByHotWord(currentHotWord, CategoryType.All)
                filterSortSearchView?.updateHotWord(
                    hotWord
                )
            }

        }

    }

    /**分类选择回调 */
    private var categoryTabSelectedCallback =
        object : CategoryTabViewLayout.OnCategoryTabSelectedListener {
            override fun onCurrentSelected(categoryType: CategoryType) {
                if (viewModel.recipesType == RecipesType.ONLINE) {
                    viewModel.getOnLineRecipeList("", currentHotWord)
                } else {
                    viewModel.queryRecipesByHotWord(currentHotWord, categoryType)
                }
            }

        }


    /**banner点击回调 */
    private var bannerItemClickCallback = object : HomeBannerAdapter.OnBannerItemClickListener {
        override fun onNetRecipesClick(bannerModel: HomeBannerModel) {
            viewModel.apply {
                getOnLineCategoryList()
                recipesType = RecipesType.ONLINE
                netFoodLiveData.postValue(true)
                postEmptyOnlineRecipes()
            }
        }
    }


    /**绑定过滤或排序内容*/
    private fun bindFilterSortView(
        adapter: BindingAdapter,
        binder: BindingAdapter.BindingViewHolder,
        name: String,
        isFilter: Boolean
    ) {
        var selectionPos = 0
        binder.apply {
            val binding = ItemFilterAndSortViewBinding.bind(itemView)
            binding.tvName.updateText(name)
            if (selectionPos == absoluteAdapterPosition) {
                binding.tvName.setTextColor(context.resources.getColor(checkedColor))
                binding.ivCheck.setBackgroundResource(icChecked)
            } else {
                binding.tvName.setTextColor(context.resources.getColor(unCheckedColor))
                binding.ivCheck.setBackgroundResource(0)
            }
            binding.itemLayout.setOnClickListener {
                val tempPosition = selectionPos
                selectionPos = absoluteAdapterPosition
                adapter.notifyItemChanged(selectionPos)
                adapter.notifyItemChanged(tempPosition)
                if (isFilter) {
                    updateCategoryName(categoryDataList[absoluteAdapterPosition])
                    filterPopupWindow.dismiss()
                } else {
                    updateSortName(absoluteAdapterPosition)
                    sortPopupWindow.dismiss()
                }
            }
        }
    }


    /**liveData回调*/
    private fun initLiveDataCallback() {
        viewModel.apply {
            //从网络菜谱切回本地菜谱，更新数据
            refreshRecipesLiveData.observe(viewLifecycleOwner) {
                viewModel.queryRecipesCategory()
            }
            //获取本地菜谱分类回调
            localCategoryLiveData.observe(viewLifecycleOwner) {
                categoryDataList = it
                //默认All
                categoryDataList.add(
                    0, DevRecipeCategory(
                        "", "", getString(com.develop.common.R.string.all), null, null, null
                    )
                )
                updateCategoryName(categoryDataList[0])
                updateSortName(0, false)
                filterRv?.models = categoryDataList
                binding.rv.scrollToPosition(0)
            }

            //获取对应分类菜谱回调
            localRecipesLiveData.observe(viewLifecycleOwner) {
                binding.page.apply {
                    finishRefresh()
                }
                binding.rv.apply {
                    models = it
                    scrollToPosition(0)
                }
            }

            //获取线上菜谱列表回调
            onLineRecipesLiveData.observe(viewLifecycleOwner) {
                binding.page.apply {
                    finishRefresh()
                }
                binding.rv.apply {
                    models = it
                    scrollToPosition(0)
                }
            }

            //获取线上分类回调
            onlineCategoryLiveData.observe(viewLifecycleOwner) {
                categoryDataList = it
                categoryDataList.add(
                    0, DevRecipeCategory(
                        "", "", getString(com.develop.common.R.string.all), null, null, null
                    )
                )
                updateCategoryName(categoryDataList[0])
                updateSortName(0, false)
                filterRv?.models = categoryDataList
                binding.rv.scrollToPosition(0)
            }

            //获取线上热门标签回调
            onLineHotTagsLiveData.observe(viewLifecycleOwner) {
                binding.rv.models = DataFactory.genOnLineHotTags(it)
            }

            //搜索回调
            hotWordSearchRecipesLiveData.observe(viewLifecycleOwner) {
                binding.rv.apply {
                    models = it
                    scrollToPosition(0)
                }
            }
        }
    }


    /**更新分类名称并获取对应数据*/
    private fun updateCategoryName(devRecipeCategory: DevRecipeCategory) {
        viewModel.currCategoryName = devRecipeCategory.name ?: ""
        categoryCode = devRecipeCategory.code
        filterSortSearchView?.updateFilterName(
            viewModel.currCategoryName
        )
        if (viewModel.recipesType == RecipesType.ONLINE) {
            viewModel.getOnLineRecipeList(categoryCode, "")
        } else {
            viewModel.queryLocalRecipes(categoryCode)
        }
    }

    /**更新分类排序并获取对应数据*/
    private fun updateSortName(pos: Int, needRefresh: Boolean = true) {
        filterSortSearchView?.updateSortName(
            sortDataList[pos].name
        )
        viewModel.sortedType = when (pos) {
            0 -> {
                SortedType.Popular
            }
            1 -> {
                SortedType.Newest
            }
            2 -> {
                SortedType.Score
            }
            else -> {
                SortedType.Popular
            }
        }
        if (needRefresh) {
            if (viewModel.recipesType == RecipesType.ONLINE) {
                viewModel.getOnLineRecipeList(categoryCode, "")
            } else {
                viewModel.queryLocalRecipes(categoryCode)
            }
        }
    }

    companion object {
        val HOME_TITLE_PROVIDER = R.layout.home_title_provider
        val HOME_FOOD_PROVIDER = R.layout.food_content_provider
        val HOME_BANNER_PROVIDER = R.layout.home_banner_provider
        val HOME_SEARCH_PROVIDER = R.layout.home_common_search_provider
        val HOME_CATEGORY_PROVIDER = R.layout.home_category_tab_provider
        val HOME_FUNCTION_PROVIDER = R.layout.home_function_provider

        var dp5 = (com.develop.base.R.dimen.convert_5px).resId2Dimension().toInt()
        var dp65 = (com.develop.base.R.dimen.convert_65px).resId2Dimension().toInt()
        var dp626 = (com.develop.base.R.dimen.convert_626px).resId2Dimension().toInt()
        var dp768 = (com.develop.base.R.dimen.convert_768px).resId2Dimension().toInt()
        var icChecked = com.develop.common.R.drawable.ic_check
        var checkedColor = com.develop.common.R.color.color_F4A12E
        var unCheckedColor = com.develop.common.R.color.color_6B6B6B
    }
}