package com.develop.step.ui

import android.graphics.Rect
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.develop.base.ext.resId2Dimension
import com.develop.common.data_repo.db.FoodContentModel
import com.develop.common.event.RefreshDataEvent
import com.develop.common.router.Screens
import com.develop.common.tag.FOOD_LIST_TYPE
import com.develop.common.tag.FOOL_LIST_TITLE
import com.develop.common.tag.Favourite
import com.develop.common.ui.CommonBVMActivity
import com.develop.common.widget.FoodContentView
import com.develop.step.R
import com.develop.step.databinding.ActivityFoodListBinding
import com.develop.step.viewmodel.FoodListViewModel
import com.drake.brv.utils.linear
import com.drake.brv.utils.models
import com.drake.brv.utils.setup
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe

@Route(path = Screens.Cook.FOOD_LIST)
class FoodListActivity : CommonBVMActivity<ActivityFoodListBinding, FoodListViewModel>() {
    private var title: String = ""

    override fun createViewModel(): FoodListViewModel {
        return getViewModel(FoodListViewModel::class.java)
    }

    override fun createViewBinding(inflater: LayoutInflater): ActivityFoodListBinding {
        return ActivityFoodListBinding.inflate(inflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        EventBus.getDefault().register(this)
        val bundle = intent.extras
        title = bundle?.getString(FOOL_LIST_TITLE) ?: ""
        viewModel.foodListType = bundle?.getInt(FOOD_LIST_TYPE) ?: Favourite
        binding.topBar.apply {
            setTopBarTitle(title)
            listenBackClick {
                finish()
            }
        }

        binding.foodListRecycler.apply {
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)
                    val pos = parent.getChildAdapterPosition(view)
                    if (pos == 0) {
                        outRect.top = dp30
                    }
                }
            })
            linear()
            setup {
                addType<FoodContentModel>(R.layout.food_content_provider)
                onBind {
                    val foodContentView =
                        findView<FoodContentView>(R.id.food_content_view)
                    foodContentView.setFoodContent(getModel(), true)
                }
            }
        }.models = viewModel.foodListLiveData.value

        viewModel.foodListLiveData.observe(this) {
            binding.foodListRecycler.models = it
        }
    }

    @Subscribe
    fun onRefreshDataEvent(event: RefreshDataEvent) {
        viewModel.refreshData()
    }

    override fun onResume() {
        super.onResume()
        viewModel.refreshData()
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
    }

    companion object {
        var dp30 = (com.develop.base.R.dimen.convert_30px).resId2Dimension().toInt()
    }
}