package com.develop.foodcooking

import android.content.Context
import com.develop.common.data_repo.FoodDataProvider
import com.develop.common.data_repo.net.Api
import com.develop.common.data_repo.net.converter.SerializationConverter
import com.develop.common.data_repo.net.interceptor.FoodRequestInterceptor
import com.develop.base.app.BaseApp
import com.develop.common.food_sdk.GlobalDevEvent
import com.develop.common.food_sdk.SerialPortUtils
import com.drake.net.convert.NetConverter
import com.drake.net.interceptor.RequestInterceptor
import com.drake.net.interfaces.NetErrorHandler
import com.kuyuntech.cofarcooking.device.sdk.eventbus.event.DevStatusEvent
import com.kuyuntech.cofarcooking.device.sdk.util.core.CofarSDK
import org.greenrobot.eventbus.Subscribe

class FoodCookingApp : BaseApp() {
    override fun onCreate() {
        super.onCreate()
        CofarSDK.register(this)
        SerialPortUtils.init()
    }

    override fun createHostUrl(): String {
        return Api.HOST
    }

    override fun createRequestInterceptor(): RequestInterceptor {
        return FoodRequestInterceptor()
    }

    override fun createConverter(): NetConverter {
        return SerializationConverter()
    }

    override fun createErrorHandle(): NetErrorHandler {
        return NetErrorHandler.DEFAULT
    }

    override fun attachBaseContext(base: Context?) {
        super.attachBaseContext(base)
        FoodDataProvider.prepareData(this)
    }


    override fun onTerminate() {
        super.onTerminate()
        SerialPortUtils.destroy()
        CofarSDK.unregister(this)
    }

    @Subscribe
    fun globalCoverEvent(event: DevStatusEvent) {
        GlobalDevEvent.globalCoverEvent(event)
    }
}